@@ -15,42 +15,50 @@
 e.g .Sort(sequence, (("akey", "nocase"), ("anotherkey", "cmp", "desc")))
 """
 
-try:
-    cmp = cmp
-except NameError: #pragma NO COVER Py3k
-    def cmp(lhs, rhs):
-        return int(rhs < lhs) - int(lhs < rhs)
+from functools import cmp_to_key
+from locale import strcoll
 
-class _Smallest(object):
+
+def cmp(lhs, rhs):
+    if lhs is None:
+        if rhs is None:
+            return 0
+        else:
+            return -1
+    elif rhs is None:
+        return 1
+    else:
+        return (lhs > rhs) - (rhs > lhs)
+
+
+class _Smallest:
     """ Singleton:  sorts below any other value.
     """
     __slots__ = ()
+
     def __lt__(self, other):
         return True
+
     def __eq__(self, other):
         return other is self
+
     def __gt__(self, other):
         return False
-_Smallest = _Smallest()
 
-try:
-    # Python 3.2+
-    from functools import cmp_to_key
-except ImportError: #pragma NO COVER Python 2.x
-    cmp_to_key = None
+
+_Smallest = _Smallest()
 
 
 def sort(sequence, sort=(), _=None, mapping=0):
     """Return a sorted copy of 'sequence'.
 
-    - sequence is a sequence of objects to be sorted
-
-    - sort is a sequence of tuples (key,func,direction)
+    :param sequence: is a sequence of objects to be sorted
+    :param sort: is a sequence of tuples (key,func,direction)
       that define the sort order:
 
-      - key is the name of an attribute to sort the objects by
+      - *key* is the name of an attribute to sort the objects by
 
-      - func is the name of a comparison function. This parameter is
+      - *func* is the name of a comparison function. This parameter is
         optional
 
         allowed values:
@@ -66,34 +74,34 @@ def sort(sequence, sort=(), _=None, mapping=0):
 
         - "xxx" -- a user-defined comparison function
 
-      - direction -- defines the sort direction for the key (optional).
+      - *direction* defines the sort direction for the key (optional).
         (allowed values: "asc" (default) , "desc")
     """
     need_sortfunc = 0
 
     if sort:
         for s in sort:
-            if len(s) > 1: # extended sort if there is reference to...
+            if len(s) > 1:  # extended sort if there is reference to...
                 # ...comparison function or sort order, even if they are
                 # "cmp" and "asc"
                 need_sortfunc = 1
                 break
 
-    sortfields = sort # multi sort = key1,key2
-    multsort = len(sortfields) > 1 # flag: is multiple sort
+    sortfields = sort  # multi sort = key1,key2
+    multsort = len(sortfields) > 1  # flag: is multiple sort
 
     if need_sortfunc:
         # prepare the list of functions and sort order multipliers
         sf_list = make_sortfunctions(sortfields, _)
 
         # clean the mess a bit
-        if multsort: # More than one sort key.
+        if multsort:  # More than one sort key.
             sortfields = [x[0] for x in sf_list]
         else:
             sort = sf_list[0][0]
 
     elif sort:
-        if multsort: # More than one sort key.
+        if multsort:  # More than one sort key.
             sortfields = [x[0] for x in sort]
         else:
             sort = sort[0][0]
@@ -113,7 +121,7 @@ def sort(sequence, sort=(), _=None, mapping=0):
             v = client
 
         if sort:
-            if multsort: # More than one sort key.
+            if multsort:  # More than one sort key.
                 k = []
                 for sk in sortfields:
                     try:
@@ -124,13 +132,13 @@ def sort(sequence, sort=(), _=None, mapping=0):
                     except (AttributeError, KeyError):
                         akey = _Smallest
                     else:
-                        if type(akey) not in BASIC_TYPES:
+                        if not isinstance(akey, BASIC_TYPES):
                             try:
                                 akey = akey()
-                            except:
+                            except BaseException:  # pylint:disable=bare-except
                                 pass
                     k.append(akey)
-            else: # One sort key.
+            else:  # One sort key.
                 try:
                     if mapping:
                         k = v[sort]
@@ -138,20 +146,17 @@ def sort(sequence, sort=(), _=None, mapping=0):
                         k = getattr(v, sort)
                 except (AttributeError, KeyError):
                     k = _Smallest
-                if type(k) not in BASIC_TYPES:
+                if not isinstance(k, BASIC_TYPES):
                     try:
                         k = k()
-                    except:
+                    except BaseException:  # pylint:disable=bare-except
                         pass
 
-        s.append((k,client))
+        s.append((k, client))
 
     if need_sortfunc:
         by = SortBy(multsort, sf_list)
-        if cmp_to_key is None:
-            s.sort(by)
-        else: #pragma NO COVER Py3k
-            s.sort(key=cmp_to_key(by))
+        s.sort(key=cmp_to_key(by))
     else:
         s.sort()
 
@@ -160,38 +165,40 @@ def sort(sequence, sort=(), _=None, mapping=0):
 
 SortEx = sort
 
-BASIC_TYPES = {
-    type(''): 1,
-    type(b''): 1,
-    type(0): 1,
-    type(0.0): 1,
-    type(()): 1,
-    type([]): 1,
-    type(None) : 1,
-}
+BASIC_TYPES = (
+    str,
+    bytes,
+    int,
+    float,
+    tuple,
+    list,
+    type(None)
+)
 
-try:
-    unicode
-except NameError: #pragma NO COVER Py3k
-    pass
-else: #pragma NO COVER Python 2
-    BASIC_TYPES[unicode] = 1
 
 def nocase(str1, str2):
     return cmp(str1.lower(), str2.lower())
 
-import sys
-if "locale" in sys.modules: # only if locale is already imported
-    from locale import strcoll
 
-    def strcoll_nocase(str1, str2): #pragma NO COVER XXX global locale
-        return strcoll(str1.lower(), str2.lower())
+def strcoll_nocase(str1, str2):
+    return strcoll(str1.lower(), str2.lower())
+
+
+_SORT_FUNCTIONS = {
+    "cmp": cmp,
+    "nocase": nocase,
+    "locale": strcoll,
+    "strcoll": strcoll,
+    "locale_nocase": strcoll_nocase,
+    "strcoll_nocase": strcoll_nocase,
+}
 
 
 def make_sortfunctions(sortfields, _):
     """Accepts a list of sort fields; splits every field, finds comparison
-    function. Returns a list of 3-tuples (field, cmp_function, asc_multplier)"""
-
+    function. Returns a list of 3-tuples (field, cmp_function, asc_multplier)
+    """
+    # pylint:disable=too-many-branches
     sf_list = []
     for field in sortfields:
         info = list(field)
@@ -211,15 +218,9 @@ def make_sortfunctions(sortfields, _):
         f_name = info[1]
 
         # predefined function?
-        if f_name == "cmp":
-            func = cmp # builtin
-        elif f_name == "nocase":
-            func = nocase
-        elif f_name in ("locale", "strcoll"): #pragma NO COVER
-            func = strcoll
-        elif f_name in ("locale_nocase", "strcoll_nocase"): #pragma NO COVER
-            func = strcoll_nocase
-        else: # no - look it up in the namespace
+        func = _SORT_FUNCTIONS.get(f_name)
+        # no - look it up in the namespace
+        if func is None:
             if hasattr(_, 'getitem'):
                 # support for zope.documenttemplate.dt_util.TemplateDict
                 func = _.getitem(f_name, 0)
@@ -241,7 +242,7 @@ def make_sortfunctions(sortfields, _):
     return sf_list
 
 
-class SortBy(object):
+class SortBy:
     def __init__(self, multsort, sf_list):
         self.multsort = multsort
         self.sf_list = sf_list
@@ -249,13 +250,12 @@ class SortBy(object):
     def __call__(self, o1, o2):
         n_fields = len(self.sf_list)
         if self.multsort:
-            o1 = o1[0] # if multsort - take the first element (key list)
+            o1 = o1[0]  # if multsort - take the first element (key list)
             o2 = o2[0]
             req_len = n_fields
         else:
             req_len = n_fields + 1
 
-
         # assert that o1 and o2 are tuples of apropriate length
         if len(o1) != req_len:
             raise ValueError("%s, %d" % (o1, req_len))
@@ -269,6 +269,12 @@ class SortBy(object):
             # if not multsort - i is 0, and the 0th element is the key
             c1, c2 = o1[i], o2[i]
             func, multiplier = self.sf_list[i][1:3]
+            if c1 is _Smallest and c2 is _Smallest:
+                return 0
+            if c1 is _Smallest:
+                return -1
+            elif c2 is _Smallest:
+                return 1
             n = func(c1, c2)
             if n:
                 return n * multiplier
