@@ -13,20 +13,29 @@
 import unittest
 
 
-def _skip_on_Py3k(func):
-    import functools
-    import sys
-    if sys.version_info[0] > 2:
-        def _dummy(self):
-            pass
-        return functools.update_wrapper(_dummy, func)
-    return func
+# we have a lot of "foo" and "bar"
+# pylint:disable=blacklisted-name
+# pylint:disable=protected-access
 
+# for sorting objects that are sortable but do not have any other attributes
+# or string-like behavior
+
+
+class _Broken:
+    def __lt__(self, other):
+        return True
+
+    def __eq__(self, other):
+        return other is self
+
+
+class _OK:
+    def __init__(self, _id):
+        self.id = _id
 
 
 class Test_sort(unittest.TestCase):
-    """Test zope.sequencesort.sort()
-    """
+
     def _callFUT(self, *args, **kw):
         from zope.sequencesort.ssort import sort
         return sort(*args, **kw)
@@ -36,7 +45,7 @@ class Test_sort(unittest.TestCase):
         self.assertEqual(self._callFUT(TUPLES), sorted(TUPLES))
 
     def test_w_attributes(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, bar):
                 self.bar = bar
         TO_SORT = [Foo('b'), Foo('a'), Foo('c')]
@@ -44,63 +53,72 @@ class Test_sort(unittest.TestCase):
         self.assertEqual([x.bar for x in result], ['a', 'b', 'c'])
 
     def test_w_attributes_nocase(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, bar):
                 self.bar = bar
         TO_SORT = [Foo('b'), Foo('A'), Foo('C')]
         result = self._callFUT(TO_SORT, (('bar', 'nocase'),))
         self.assertEqual([x.bar for x in result], ['A', 'b', 'C'])
 
+    def test_w_attributes_strcoll_nocase(self):
+        class Foo:
+            def __init__(self, bar):
+                self.bar = bar
+        TO_SORT = [Foo('b'), Foo('A'), Foo('C')]
+        result = self._callFUT(TO_SORT, (('bar', 'strcoll_nocase'),))
+        self.assertEqual([x.bar for x in result], ['A', 'b', 'C'])
+
     def test_w_attributes_missing(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, bar):
                 self.bar = bar
         TO_SORT = [Foo('b'), Foo('a'), Foo('c'), object()]
         result = self._callFUT(TO_SORT, (('bar',),))
         self.assertEqual([getattr(x, 'bar', 'ZZZ') for x in result],
-                          ['ZZZ', 'a', 'b', 'c'])
+                         ['ZZZ', 'a', 'b', 'c'])
 
     def test_w_multi_attributes(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, bar, baz):
                 self.bar = bar
                 self.baz = baz
         TO_SORT = [Foo('b', 'q'), Foo('a', 'r'), Foo('c', 's'), Foo('a', 'p')]
         result = self._callFUT(TO_SORT, (('bar',), ('baz',)))
         self.assertEqual([(x.bar, x.baz) for x in result],
-                          [('a', 'p'), ('a', 'r'), ('b', 'q'), ('c', 's')])
+                         [('a', 'p'), ('a', 'r'), ('b', 'q'), ('c', 's')])
 
     def test_w_multi_attributes_nocase(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, bar, baz):
                 self.bar = bar
                 self.baz = baz
         TO_SORT = [Foo('b', 'q'), Foo('a', 'R'), Foo('c', 's'), Foo('a', 'p')]
         result = self._callFUT(TO_SORT, (('bar',), ('baz', 'nocase'),))
         self.assertEqual([(x.bar, x.baz) for x in result],
-                          [('a', 'p'), ('a', 'R'), ('b', 'q'), ('c', 's')])
+                         [('a', 'p'), ('a', 'R'), ('b', 'q'), ('c', 's')])
 
     def test_w_multi_attributes_missing(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, bar, baz):
                 self.bar = bar
                 self.baz = baz
         TO_SORT = [Foo('b', 'q'), Foo('a', 'r'), object(), Foo('a', 'p')]
         result = self._callFUT(TO_SORT, (('bar',), ('baz',)))
         self.assertEqual([(getattr(x, 'bar', 'ZZZ'), getattr(x, 'baz', 'YYY'))
-                                for x in result],
-                          [('ZZZ', 'YYY'), ('a', 'p'), ('a', 'r'), ('b', 'q')])
+                          for x in result],
+                         [('ZZZ', 'YYY'), ('a', 'p'), ('a', 'r'), ('b', 'q')])
 
     def test_w_non_basictype_key(self):
-        from zope.sequencesort.ssort import cmp
-        class Qux(object):
+        from zope.sequencesort.ssort import cmp as compare
+
+        class Qux:
             def __init__(self, spam):
                 self._spam = spam
-            def __cmp__(self, other):
-                return cmp(self._spam, other._spam)
+
             def __lt__(self, other):
-                return self._spam < other._spam
-        class Foo(object):
+                return compare(self._spam, other._spam) < 0
+
+        class Foo:
             def __init__(self, bar):
                 self.bar = Qux(bar)
         TO_SORT = [Foo('b'), Foo('a'), Foo('c')]
@@ -108,9 +126,10 @@ class Test_sort(unittest.TestCase):
         self.assertEqual([x.bar._spam for x in result], ['a', 'b', 'c'])
 
     def test_w_methods(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, bar):
                 self._bar = bar
+
             def bar(self):
                 return self._bar
         TO_SORT = [Foo('b'), Foo('a'), Foo('c')]
@@ -118,10 +137,11 @@ class Test_sort(unittest.TestCase):
         self.assertEqual([x.bar() for x in result], ['a', 'b', 'c'])
 
     def test_w_attribute_and_methods(self):
-        class Foo(object):
+        class Foo:
             def __init__(self, bar, baz):
                 self._bar = bar
                 self.baz = baz
+
             def bar(self):
                 return self._bar
         TO_SORT = [Foo('b', 'Q'), Foo('b', 'p'), Foo('c', 'r')]
@@ -130,15 +150,16 @@ class Test_sort(unittest.TestCase):
                          [('b', 'p'), ('b', 'Q'), ('c', 'r')])
 
     def test_w_multi_and_non_basictype_key(self):
-        from zope.sequencesort.ssort import cmp
-        class Qux(object):
+        from zope.sequencesort.ssort import cmp as compare
+
+        class Qux:
             def __init__(self, spam):
                 self._spam = spam
-            def __cmp__(self, other):
-                return cmp(self._spam, other._spam)
+
             def __lt__(self, other):
-                return self._spam < other._spam
-        class Foo(object):
+                return compare(self._spam, other._spam) < 0
+
+        class Foo:
             def __init__(self, bar, baz):
                 self.bar = bar
                 self.baz = Qux(baz)
@@ -147,9 +168,9 @@ class Test_sort(unittest.TestCase):
         self.assertEqual([(x.bar, x.baz._spam) for x in result],
                          [('b', 'p'), ('b', 'q'), ('c', 'r')])
 
-    @_skip_on_Py3k
     def test_wo_args(self):
-        self.assertEqual(self._callFUT(WORDLIST), RES_WO_ARGS)
+        with self.assertRaises(TypeError):
+            self._callFUT(WORDLIST)
 
     def test_w_only_key(self):
         self.assertEqual(self._callFUT(WORDLIST, (("key",),), mapping=1),
@@ -168,45 +189,99 @@ class Test_sort(unittest.TestCase):
                                        mapping=1), RES_W_MULTI_KEY)
 
     def test_w_multi_key_nocase_desc(self):
-        self.assertEqual(self._callFUT(WORDLIST, (("weight",),
-                                       ("key", "nocase", "desc")), mapping=1),
-                         RES_W_MULTI_KEY_NOCASE_DESC)
+        self.assertEqual(
+            self._callFUT(WORDLIST, (("weight",),
+                                     ("key", "nocase", "desc")), mapping=1),
+            RES_W_MULTI_KEY_NOCASE_DESC)
 
     def test_w_custom_comparator(self):
-        from zope.sequencesort.ssort import cmp
-        def myCmp(s1, s2):
-            return -cmp(s1, s2)
+        from zope.sequencesort.ssort import cmp as compare
 
-        md = {"myCmp" : myCmp}
-        self.assertEqual(self._callFUT(WORDLIST,
-                                (("weight",), ("key", "myCmp", "desc")),
-                                md,
-                                mapping=1
-                                ), RES_W_CUSTOM_COMPARATOR)
+        def myCmp(s1, s2):
+            return -compare(s1, s2)
+
+        md = {"myCmp": myCmp}
+        self.assertEqual(
+            self._callFUT(
+                WORDLIST,
+                (("weight",), ("key", "myCmp", "desc")),
+                md,
+                mapping=1
+            ),
+            RES_W_CUSTOM_COMPARATOR)
 
     def test_w_custom_comparator_dtml_namespace(self):
-        from zope.sequencesort.ssort import cmp
-        class Namespace(object):
+        from zope.sequencesort.ssort import cmp as compare
+
+        class Namespace:
             def __init__(self, **kw):
                 self.__dict__.update(kw)
+
             def getitem(self, name, default):
                 return getattr(self, name, default)
 
         def myCmp(s1, s2):
-            return -cmp(s1, s2)
+            return -compare(s1, s2)
 
         ns = Namespace(myCmp=myCmp)
 
-        self.assertEqual(self._callFUT(WORDLIST,
-                                (("weight",), ("key", "myCmp", "desc")),
-                                ns,
-                                mapping=1
-                                ), RES_W_CUSTOM_COMPARATOR)
+        self.assertEqual(
+            self._callFUT(
+                WORDLIST,
+                (("weight",), ("key", "myCmp", "desc")),
+                ns,
+                mapping=1
+            ),
+            RES_W_CUSTOM_COMPARATOR)
+
+    def test_w_sort_broken(self):
+        _broken = _Broken()
+        _ok = _OK('test')
+        self.assertEqual(
+            self._callFUT([_ok, _broken]),
+            [_broken, _ok]
+        )
+
+    def test_w_sort_broken_with_key(self):
+        _broken = _Broken()
+        _ok = _OK('test')
+        self.assertEqual(
+            self._callFUT([_ok, _broken], [('id',), ]),
+            [_broken, _ok]
+        )
+
+    def test_w_sort_broken_with_key_locale(self):
+        # testing actual unicode literals to be sorted correctly has the
+        # problem that one can not assume that there is any locale actually
+        # installed on the system. But we can test if a broken object can be
+        # compared with a correct one even for a sorting that assumes stringy
+        # behavior
+        _broken = _Broken()
+        v1 = _OK('test')
+        self.assertEqual(
+            self._callFUT([v1, _broken], [('id', 'locale',), ]),
+            [_broken, v1]
+        )
+        self.assertEqual(
+            self._callFUT([_broken, v1], [('id', 'locale',), ]),
+            [_broken, v1]
+        )
+
+    def test_w_sort_broken_with_key_locale_nocase(self):
+        _broken = _Broken()
+        v1 = _OK('test')
+        self.assertEqual(
+            self._callFUT([v1, _broken], [('id', 'locale_nocase',), ]),
+            [_broken, v1]
+        )
+        self.assertEqual(
+            self._callFUT([_broken, v1], [('id', 'locale_nocase',), ]),
+            [_broken, v1]
+        )
 
 
 class Test_make_sortfunctions(unittest.TestCase):
-    """Test zope.sequencesort.sort()
-    """
+
     def _callFUT(self, sortfields, _):
         from zope.sequencesort.ssort import make_sortfunctions
         return make_sortfunctions(sortfields, _)
@@ -223,6 +298,7 @@ class Test_make_sortfunctions(unittest.TestCase):
 class SortByTests(unittest.TestCase):
     """Test zope.sequencesort.sort()
     """
+
     def _getTargetClass(self):
         from zope.sequencesort.ssort import SortBy
         return SortBy
@@ -230,14 +306,9 @@ class SortByTests(unittest.TestCase):
     def _makeOne(self, multisort, sf_list):
         return self._getTargetClass()(multisort, sf_list)
 
-    def _makeField(self, name, _cmp=None, multiplier=1):
-        if _cmp is None:
-            try:
-                _cmp = cmp
-            except NameError: #pragma NO COVER Py3k
-                def _cmp(lhs, rhs):
-                    return int(rhs < lhs) - int(lhs < rhs)
-        return (name, _cmp, multiplier)
+    def _makeField(self, name, multiplier=1):
+        from zope.sequencesort.ssort import cmp
+        return (name, cmp, multiplier)
 
     def test_invalid_length_single(self):
         sb = self._makeOne(False, [self._makeField('bar')])
@@ -277,34 +348,51 @@ class SortByTests(unittest.TestCase):
         self.assertEqual(sb((['a', 'q'], None), (['a', 'r'], None)), 1)
         self.assertEqual(sb((['b', 'p'], None), (['a', 'q'], None)), 1)
 
+    def test_smallest(self):
+        from zope.sequencesort.ssort import _Smallest
+        sb = self._makeOne(False, [self._makeField('bar')])
+        self.assertEqual(sb(('a', None), (_Smallest, None)), 1)
+        self.assertEqual(sb((_Smallest, None), ('a', None)), -1)
+        self.assertEqual(sb((_Smallest, None), (_Smallest, None)), 0)
+
+    def test_none(self):
+        sb = self._makeOne(False, [self._makeField('bar')])
+        self.assertEqual(sb((None, None), ('a', None)), -1)
+        self.assertEqual(sb((None, None), (100, None)), -1)
+        self.assertEqual(sb((None, None), (1.0, None)), -1)
+        self.assertEqual(sb((None, None), (None, None)), 0)
+        self.assertEqual(sb(('a', None), (None, None)), 1)
+        self.assertEqual(sb((100, None), (None, None)), 1)
+        self.assertEqual(sb((1.0, None), (None, None)), 1)
+
 
 WORDLIST = [
-   {"key": "aaa", "word": "AAA", "weight": 1},
-   {"key": "bbb", "word": "BBB", "weight": 0},
-   {"key": "ccc", "word": "CCC", "weight": 0},
-   {"key": "ddd", "word": "DDD", "weight": 0},
-   {"key": "eee", "word": "EEE", "weight": 1},
-   {"key": "fff", "word": "FFF", "weight": 0},
-   {"key": "ggg", "word": "GGG", "weight": 0},
-   {"key": "hhh", "word": "HHH", "weight": 0},
-   {"key": "iii", "word": "III", "weight": 1},
-   {"key": "jjj", "word": "JJJ", "weight": -1},
-   {"key": "kkk", "word": "KKK", "weight": 0},
-   {"key": "lll", "word": "LLL", "weight": 0},
-   {"key": "mmm", "word": "MMM", "weight": 0},
-   {"key": "nnn", "word": "NNN", "weight": 0},
-   {"key": "ooo", "word": "OOO", "weight": 1},
-   {"key": "ppp", "word": "PPP", "weight": 0},
-   {"key": "qqq", "word": "QQQ", "weight": -1},
-   {"key": "rrr", "word": "RRR", "weight": 0},
-   {"key": "sss", "word": "SSS", "weight": 0},
-   {"key": "ttt", "word": "TTT", "weight": 0},
-   {"key": "uuu", "word": "UUU", "weight": 1},
-   {"key": "vvv", "word": "VVV", "weight": 0},
-   {"key": "www", "word": "WWW", "weight": 0},
-   {"key": "xxx", "word": "XXX", "weight": 0},
-   {"key": "yyy", "word": "YYY", "weight": -1},
-   {"key": "zzz", "word": "ZZZ", "weight": 0}
+    {"key": "aaa", "word": "AAA", "weight": 1},
+    {"key": "bbb", "word": "BBB", "weight": 0},
+    {"key": "ccc", "word": "CCC", "weight": 0},
+    {"key": "ddd", "word": "DDD", "weight": 0},
+    {"key": "eee", "word": "EEE", "weight": 1},
+    {"key": "fff", "word": "FFF", "weight": 0},
+    {"key": "ggg", "word": "GGG", "weight": 0},
+    {"key": "hhh", "word": "HHH", "weight": 0},
+    {"key": "iii", "word": "III", "weight": 1},
+    {"key": "jjj", "word": "JJJ", "weight": -1},
+    {"key": "kkk", "word": "KKK", "weight": 0},
+    {"key": "lll", "word": "LLL", "weight": 0},
+    {"key": "mmm", "word": "MMM", "weight": 0},
+    {"key": "nnn", "word": "NNN", "weight": 0},
+    {"key": "ooo", "word": "OOO", "weight": 1},
+    {"key": "ppp", "word": "PPP", "weight": 0},
+    {"key": "qqq", "word": "QQQ", "weight": -1},
+    {"key": "rrr", "word": "RRR", "weight": 0},
+    {"key": "sss", "word": "SSS", "weight": 0},
+    {"key": "ttt", "word": "TTT", "weight": 0},
+    {"key": "uuu", "word": "UUU", "weight": 1},
+    {"key": "vvv", "word": "VVV", "weight": 0},
+    {"key": "www", "word": "WWW", "weight": 0},
+    {"key": "xxx", "word": "XXX", "weight": 0},
+    {"key": "yyy", "word": "YYY", "weight": -1},
+    {"key": "zzz", "word": "ZZZ", "weight": 0}
 ]
 
 RES_WO_ARGS = [
@@ -512,8 +600,4 @@ RES_W_CUSTOM_COMPARATOR = [
 
 
 def test_suite():
-    return unittest.TestSuite((
-        unittest.makeSuite(Test_sort),
-        unittest.makeSuite(Test_make_sortfunctions),
-        unittest.makeSuite(SortByTests),
-    ))
+    return unittest.defaultTestLoader.loadTestsFromName(__name__)
